<?php

class KAClient {

    const SUCCESS_CODE = 100;
    const ERROR_CODE_NOT_FOUND = 220;
    const ERROR_CODE_INCORRECT_FORMAT = 410;

    public static function getKeyInfo($keyNumber)
    {
        $request =
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>
            <methodCall>
                <methodName>partner10.getKeyInfo</methodName>
                <params>
                  <param>
                    <value>
                      <struct>
                        <member>
                          <name>login</name>
                          <value><string>".sfConfig::get('app_ka_api_login')."</string></value>
                        </member>
                        <member>
                          <name>password</name>
                          <value><string>".sfConfig::get('app_ka_api_password')."</string></value>
                        </member>
                      </struct>
                    </value>
                  </param>
                  <param>
                    <value><string>{$keyNumber}</string></value>
                  </param>
                </params>
            </methodCall>";
        $result = self::_call($request);
        return $result->xpath('//member[name="keyInfo"]/value/struct')[0];
    }

    private static function _call($request)
    {
        $handler = curl_init(sfConfig::get('app_ka_api_url'));
        if (false === $handler) {
            throw new StoreException('Failed to create cURL handler');
        }
        curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($handler, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
        curl_setopt($handler, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($handler, CURLOPT_POSTFIELDS, $request);
        curl_setopt($handler, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($handler, CURLOPT_SSL_VERIFYHOST, false);

        $response = curl_exec($handler);
        $errorCode = curl_errno($handler);
        $errorMessage = curl_error($handler);
        curl_close($handler);

        if ($errorCode) {
            throw new StoreException("cURL error {$errorCode}: {$errorMessage}");
        }

        $result = simplexml_load_string($response);
        if (isset($result->fault)) {
            $faultCode = (int)$result->xpath('//fault/value/struct/member[name="faultCode"]/value/int/text()')[0];
            $faultString = (string)$result->xpath('//fault/value/struct/member[name="faultString"]/value/text()')[0];
            throw new StoreException("KA fault {$faultCode}: {$faultString}");
        }

        $resultCode = (int)$result->xpath('//member[name="resultCode"]/value/int/text()')[0];
        if (self::SUCCESS_CODE != $resultCode) {
            switch ($resultCode) {
                case self::ERROR_CODE_NOT_FOUND:
                    throw new InputException('TRANS_ORDERFORM_PARENT_KEY_NOT_FOUND');
                case self::ERROR_CODE_INCORRECT_FORMAT:
                    throw new InputException('TRANS_ORDERFORM_PARENT_KEY_INCORRECT_FORMAT');
                default:
                    $resultDesc = (string)$result->xpath('//member[name="resultDesc"]/value/text()')[0];
                    throw new StoreException("KA error {$resultCode}: {$resultDesc}");
            }
        }
        return $result;
    }
}