<?php

abstract class PurchaseSupportAction extends CatalogAction {

    const TYPE_REINSTATE = 'reinstate';
    const TYPE_RENEW = 'renew';

    public function execute($request)
    {
        $this->container = $this->_container;

        if ($request->isMethod('post')) {
            try {
                $keyNumber = trim($request->getParameter('keyNumber'));
                if (!$keyNumber) {
                    throw new InputException('TRANS_YOU_MUST_SPECIFY_KEY_NUMBER');
                }

                $planId = $this->_getPlanId(KAClient::getKeyInfo($keyNumber));
                $plan = $this->_store->getPlan($planId);
                $price = reset($plan->getPricesByCurrency($this->_store->currencyID));

                $purchase = new Purchase($plan->getId(), $price->getCycleId(), $price->getId(), $plan->productName, $plan->productType);
                if ($coupon = $plan->getAppliedCoupon()) {
                    $purchase->setCoupon($coupon);
                }
                $purchase->addParameter('customQuestionData', $this->_getCustomQuestionData($purchase, $keyNumber));

                $cart = ResourceLocator::getInstance()->cart;
                $cartItem = new CartItem();
                $cartItem->addPurchase($purchase);
                $cart->setCurrentItem($cartItem);
                $this->_attachAffiliateProgramParams($cart);
                $this->_attachCouponParams($cart);

                $steps = array(
                    new RouterPage('add-purchase'),
                    new RouterPage('summary'),
                    new RouterPage('contact-information'),
                    new RouterPage('order-details'),
                    new RouterPage('confirmation'),
                    new RouterPage('thank-you')
                );
                $router = new Router();
                $router->init($steps, new RouterPage('catalog', array('container' => $this->_container)));

                ResourceLocator::getInstance()->routersStack->push($router);
                $router->next(array(), true, true);

                // set no cache and location headers manually, because simple $this->redirect call entail clear
                // of http headers, so some browsers may cache result of this action and prevent it execution
                sfContext::getInstance()->getConfiguration()->loadHelpers('Url');
                $response = $this->getResponse();
                $response->addCacheControlHttpHeader('no-cache');
                $response->setHttpHeader('Location', url_for(reset($steps)->getUrl()));

                // send only headers to perform redirect
                return sfView::HEADER_ONLY;
            } catch (InputException $exception) {
                $this->error = $exception->getMessage();
                $this->keyNumber = $keyNumber;
            }
        }
    }

    protected abstract function _getPlanId(SimpleXMLElement $keyInfo);

    private function _getCustomQuestionData($purchase, $keyNumber)
    {
        $response = ResourceLocator::getInstance()->billingContactService->executeGetCustomQuestions(
            null,
            array($purchase->getId() => array('planId' => $purchase->getPlanId(), 'parentPlanId' => null))
        );
        if ($response->hasErrors()) {
            throw new BillingServiceException('Can not retrieve create customer custom questions form');
        }

        $customQuestionData = array();
        foreach ($response->getData()['customQuestion']['inputs'] as $customQuestion) {
            $customQuestionData[$customQuestion['id']] = $keyNumber;
        }
        return $customQuestionData;
    }
}
