<?php

class BuySusAction extends PurchaseSupportAction {

    protected function _getPlanId(SimpleXMLElement $keyInfo)
    {
        if ('PURCHASE' != (string)$keyInfo->xpath('member[name="billingType"]/value/text()')[0]) {
            throw new InputException('TRANS_ORDERFORM_SUS_INAPPROPRIATE_KEY_TYPE');
        }

        $mapping = sfConfig::get('app_sus', array());
        $keyType = (string)$keyInfo->xpath('member[name="apiKeyType"]/value/text()')[0];

        $keyFeatures = array();
        foreach($keyInfo->xpath('member[name="features"]/value/array/data/value') as $featureNode) {
            $feature = (string)$featureNode->xpath('struct/member[name="apiName"]/value/text()')[0];
            $keyFeatures[] = str_replace('_WO_SUS', '', $feature);
        }

        $susAndSupportInfo = $keyInfo->xpath('member[name="susAndSupportInfo"]/value/array/data')[0];
        $expirationDate = (string)$susAndSupportInfo->xpath('value/struct[member[value="SUS Present"]]/member[name="expiration"]/value/dateTime.iso8601/text()')[0];
        if (!$expirationDate) {
            $planType = self::TYPE_REINSTATE;
        } else {
            $date = new DateTime($expirationDate, new DateTimeZone('UTC'));
            $date->add(new DateInterval('P1D'));
            $planType = $date > new DateTime('now', new \DateTimeZone('UTC')) ? self::TYPE_RENEW : self::TYPE_REINSTATE;
        }

        foreach ($mapping[$planType] as $planId => $configuration) {
            foreach ($configuration as $type => $features) {
                if ($type != $keyType) {
                    continue;
                }
                // key type might not require any domain features at all
                if (empty($features) || array_intersect($features, $keyFeatures)) {
                    $isProblem = (int)$keyInfo->xpath('member[name="problem"]/value/boolean/text()')[0];
                    $isTerminated = (int)$keyInfo->xpath('member[name="terminated"]/value/boolean/text()')[0];
                    if ($isProblem || $isTerminated) {
                        throw new InputException('TRANS_ORDERFORM_SUS_INACTIVE_LICENSE');
                    }
                    if (self::TYPE_REINSTATE == $planType) {
                        sfContext::getInstance()->getUser()->setAttribute('warningMessage', 'TRANS_ORDERFORM_SUS_REINSTATEMENT');
                    }
                    return $planId;
                }
            }
        }

        throw new InputException('TRANS_ORDERFORM_SUS_INAPPROPRIATE_KEY_TYPE');
    }
}
